@@ -17,9 +17,9 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-__id__        = "$Id: J2SE-access-bridge.py 3281 2007-11-29 20:05:13Z wwalker $"
-__version__   = "$Revision: 3281 $"
-__date__      = "$Date: 2007-11-29 15:05:13 -0500 (Thu, 29 Nov 2007) $"
+__id__        = "$Id: J2SE-access-bridge.py 3433 2008-01-07 18:26:04Z wwalker $"
+__version__   = "$Revision: 3433 $"
+__date__      = "$Date: 2008-01-07 13:26:04 -0500 (Mon, 07 Jan 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -201,7 +201,20 @@ class Script(default.Script):
 
         Returns True if the event is of interest.
         """
+
+        # The Java platform chooses to give us keycodes different from
+        # the native platform keycodes.  So, we hack here by converting
+        # the keysym we get from Java into a keycode.
+
         keysym = keyboardEvent.event_string
+
+        # We need to make sure we have a keysym-like thing.  The space
+        # character is not a keysym, so we convert it into the string,
+        # 'space', which is.
+        #
+        if keysym == " ":
+            keysym = "space"
+
         keyboardEvent.hw_code = keybindings.getKeycode(keysym)
         return default.Script.consumesKeyboardEvent(self, keyboardEvent)
 
