@@ -25,9 +25,9 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 3167 2007-10-23 23:01:32Z joanied $"
-__version__   = "$Revision: 3167 $"
-__date__      = "$Date: 2007-10-23 19:01:32 -0400 (Tue, 23 Oct 2007) $"
+__id__        = "$Id: braille.py 3831 2008-04-15 23:35:20Z eitani $"
+__version__   = "$Revision: 3831 $"
+__date__      = "$Date: 2008-04-15 19:35:20 -0400 (Tue, 15 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -38,6 +38,14 @@ import signal
 
 import pyatspi
 
+try:
+    import louis
+except ImportError:
+    louis = None
+    _defaultContractionTable = None
+else:
+    _defaultContractionTable = louis.getDefaultTable()
+
 # We'll use the official BrlAPI pythons (as of BrlTTY 3.8) if they
 # are available.  Otherwise, we'll fall back to our own bindings.
 #
@@ -228,7 +236,7 @@ _regionWithFocus = None
 # corner is defined by the point (x, line number).  As such, the viewport is
 # identified solely by its upper left point.
 #
-_viewport = [0, 0]
+viewport = [0, 0]
 
 # The callback to call on a BrlTTY input event.  This is passed to
 # the init method.
@@ -264,7 +272,7 @@ class Region:
     each region is determined by its string.
     """
 
-    def __init__(self, string, cursorOffset=0):
+    def __init__(self, string, cursorOffset=0, expandOnCursor=False):
         """Creates a new Region containing the given string.
 
         Arguments:
@@ -276,13 +284,27 @@ class Region:
         if not string:
             string = ""
 
+        # If louis is None, then we don't go into contracted mode.
+        self.contracted = settings.enableContractedBraille and \
+                          louis is not None
+        
+        self.expandOnCursor = expandOnCursor
+        
         string = string.decode("UTF-8")
-        if string[-1:] == "\n":
-            string = string[:-1]
-        self.string = string.encode("UTF-8")
+        string = string.strip('\n')
+        self.rawLine = string.encode("UTF-8")
 
-        self.cursorOffset = cursorOffset
+        if self.contracted:
+            self.contractionTable = settings.brailleContractionTable or \
+                                    _defaultContractionTable
 
+            self.string, self.inPos, self.outPos, self.cursorOffset = \
+                         self.contractLine(self.rawLine,
+                                           cursorOffset, expandOnCursor)
+        else:
+            self.string = self.rawLine
+            self.cursorOffset = cursorOffset
+            
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of string
@@ -299,14 +321,91 @@ class Region:
         #
         mask = ['\x00'] * len(self.string)
         return "".join(mask)
+    
+    def repositionCursor(self):
+        """Reposition the cursor offset for contracted mode.
+        """
+        if self.contracted:
+            self.string, self.inPos, self.outPos, self.cursorOffset = \
+                       self.contractLine(self.rawLine,
+                                         self.cursorOffset,
+                                         self.expandOnCursor)
 
+    def contractLine(self, line, cursorOffset=0, expandOnCursor=False):
+        """Contract the given line. Returns the contracted line, and the
+        cursor position in the contracted line.
+
+        Arguments:
+        - line: Line to contract.
+        - cursorOffset: Offset of cursor,defaults to 0.
+        - expandOnCursor: Expand word under cursor, False by default.
+        """
+
+        try:
+            cursorOnSpace = line[cursorOffset] == ' '
+        except IndexError:
+            cursorOnSpace = False
+            
+        if not expandOnCursor or cursorOnSpace:
+            contracted, inPos, outPos, cursorPos = \
+                             louis.translate([self.contractionTable],
+                                             line.decode(),
+                                             cursorPos=cursorOffset)
+        else:
+            contracted, inPos, outPos, cursorPos = \
+                             louis.translate([self.contractionTable],
+                                             line.decode(),
+                                             cursorPos=cursorOffset,
+                                             mode=louis.MODE.compbrlAtCursor)
+
+        return contracted, inPos, outPos, cursorPos
+    
+    def displayToBufferOffset(self, display_offset):
+        try:
+            offset = self.inPos[display_offset]
+        except IndexError:
+            # Off the chart, we just place the cursor at the end of the line.
+            offset = len(self.rawLine)
+        except AttributeError:
+            # Not in contracted mode.
+            offset = display_offset
+
+        return offset
+
+    def setContractedBraille(self, contracted):
+        if self.contracted == contracted:
+            return
+        self.contracted = contracted
+        if contracted:
+            self.contractionTable = settings.brailleContractionTable or \
+                                    _defaultContractionTable
+            self.contractRegion()
+        else:
+            self.expandRegion()
+
+    def contractRegion(self):
+        self.string, self.inPos, self.outPos, self.cursorOffset = \
+                     self.contractLine(self.rawLine,
+                                       self.cursorOffset,
+                                       self.expandOnCursor)
+        
+    def expandRegion(self):
+        if not self.contracted:
+            return
+        self.string = self.rawLine
+        try:
+            self.cursorOffset = self.inPos[self.cursorOffset]
+        except IndexError:
+            self.cursorOffset = len(self.string)
+        
 class Component(Region):
     """A subclass of Region backed by an accessible.  This Region will react
     to any cursor routing key events and perform the default action on the
     accessible, if a default action exists.
     """
 
-    def __init__(self, accessible, string, cursorOffset=0):
+    def __init__(self, accessible, string, cursorOffset=0,
+                 indicator='', expandOnCursor=False):
         """Creates a new Component.
 
         Arguments:
@@ -316,7 +415,13 @@ class Component(Region):
                         for this Region if it gets focus.
         """
 
-        Region.__init__(self, string, cursorOffset)
+        Region.__init__(self, string, cursorOffset, expandOnCursor)
+        if indicator:
+            if self.string:
+                self.string = indicator + ' ' + self.string
+            else:
+                self.string = indicator
+
         self.accessible = accessible
 
     def processRoutingKey(self, offset):
@@ -349,7 +454,8 @@ class Text(Region):
     [[[TODO: WDW - need to add in text selection capabilities.  Logged
     as bugzilla bug 319754.]]]"""
 
-    def __init__(self, accessible, label=None, eol=""):
+    def __init__(self, accessible, label="", eol="", 
+                 startOffset=None, endOffset=None):
         """Creates a new Text region.
 
         Arguments:
@@ -362,33 +468,41 @@ class Text(Region):
             [string, self.caretOffset, self.lineOffset] = \
                  orca_state.activeScript.getTextLineAtCaret(self.accessible)
 
-        # Sometimes, gnome-terminal will give us very odd values when
-        # the user is editing using 'vi' and has positioned the caret
-        # at the first character of the first line.  In this case, we
-        # end up getting a very large negative number for the line offset.
-        # So, we just assume the user is at the first character.
-        #
-        if self.lineOffset < 0:
-            self.caretOffset = 0
-            self.lineOffset = 0
-            [string, startOffset, endOffset] = \
-                self.accessible.queryText().getTextAtOffset(
-                    0,
-                    pyatspi.TEXT_BOUNDARY_LINE_START)
+        string = string.decode("UTF-8")
+
+        try:
+            endOffset = endOffset - self.lineOffset
+        except TypeError:
+            pass
+
+        try:
+            self.startOffset = startOffset - self.lineOffset
+        except TypeError:
+            self.startOffset = 0
+
+        string = string[self.startOffset:endOffset]
+
+        self.caretOffset -= self.startOffset
 
         cursorOffset = min(self.caretOffset - self.lineOffset, len(string))
 
         self._maxCaretOffset = self.lineOffset + len(string.decode("UTF-8"))
 
         self.eol = eol
-        string = string + self.eol
 
-        self.label = label
-        if self.label:
-            string = self.label + " " + string
-            cursorOffset += len(self.label.decode("UTF-8")) + 1
+        if label:
+            self.label = label + ' '
+        else:
+            self.label = ''
 
-        Region.__init__(self, string, cursorOffset)
+        string = self.label + string
+
+        cursorOffset += len(self.label)
+
+        Region.__init__(self, string, cursorOffset, True)
+
+        if not self.contracted:
+            self.string += self.eol
 
     def repositionCursor(self):
         """Attempts to reposition the cursor in response to a new
@@ -400,15 +514,20 @@ class Text(Region):
         [string, caretOffset, lineOffset] = \
                  orca_state.activeScript.getTextLineAtCaret(self.accessible)
         cursorOffset = min(caretOffset - lineOffset, len(string))
-        if self.label:
-            cursorOffset += len(self.label.decode("UTF-8")) + 1
-
+        
         if lineOffset != self.lineOffset:
             return False
-        else:
-            self.caretOffset = caretOffset
-            self.lineOffset = lineOffset
-            self.cursorOffset = cursorOffset
+
+        self.caretOffset = caretOffset
+        self.lineOffset = lineOffset
+
+        cursorOffset += len(self.label)
+
+        if self.contracted:
+            self.string, self.inPos, self.outPos, cursorOffset = \
+                       self.contractLine(self.rawLine, cursorOffset, True)
+
+        self.cursorOffset = cursorOffset
 
         return True
 
@@ -417,14 +536,15 @@ class Text(Region):
         is 0-based, where 0 represents the leftmost character of text
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
+        
+        offset = self.displayToBufferOffset(offset)
 
-        if self.label:
-            offset = offset - len(self.label.decode("UTF-8")) - 1
-            if offset < 0:
-                return
+        if offset < 0:
+            return
 
         newCaretOffset = min(self.lineOffset + offset, self._maxCaretOffset)
-        self.accessible.queryText().setCaretOffset(newCaretOffset)
+        orca_state.activeScript.setCaretOffset(
+            self.accessible, newCaretOffset)
 
     def getAttributeMask(self):
         """Creates a string which can be used as the attrOr field of brltty's
@@ -438,95 +558,86 @@ class Text(Region):
 
         # Start with an empty mask.
         #
-        stringLength = len(self.string) - len(self.eol)
-        attrMask = ['\x00']*stringLength
+        stringLength = len(self.rawLine) - len(self.label)
+        lineEndOffset = self.lineOffset + stringLength
+        regionMask = [settings.TEXT_ATTR_BRAILLE_NONE]*stringLength
 
         attrIndicator = settings.textAttributesBrailleIndicator
         selIndicator = settings.brailleSelectorIndicator
-        indicateAttr = (attrIndicator != settings.TEXT_ATTR_BRAILLE_NONE)
-        indicateSel = (selIndicator != settings.BRAILLE_SEL_NONE)
+        script = orca_state.activeScript
 
-        if indicateAttr and settings.enabledBrailledTextAttributes:
-            # Identify what attributes the user cares about.  Also get the
-            # default attributes for the text object because those attributes
-            # may not be included as attributes for the character.
-            #
-            script = orca_state.activeScript
-            enabledAttributes = settings.enabledBrailledTextAttributes
-            [userAttrList, userAttrDict] = \
-                script.textAttrsToDictionary(enabledAttributes)
-            defaultAttributes = text.getDefaultAttributes()
-            [defaultAttrList, defaultAttrDict] = \
-                script.textAttrsToDictionary(defaultAttributes)
+        if attrIndicator:
+            enabledAttributes = script.attribsToDictionary(
+                settings.enabledBrailledTextAttributes)
 
-            # [[[TODO - HACK - JD: The end offset returned by SO/OOo is the
-            # offset of the first character that doesn't have the attribute
-            # in question; all other apps return the offset of the last
-            # character that has that attribute]]]
-            #
-            adjustment = 0
-            appName = orca_state.locusOfFocus.getApplication().name
-            if appName.startswith("soffice"):
-                adjustment += 1
-
-            i = self.lineOffset
-            endOffset = self.lineOffset + stringLength
-            while i < endOffset - 1:
-                for attribute in userAttrList:
-                    defaultValue = None
-                    if attribute in defaultAttrList:
-                        defaultValue = defaultAttrDict[attribute]
-                    [value, start, end, defined] = \
-                            text.getAttributeValue(i, attribute)
-
-                    notOfInterest = userAttrDict[attribute]
-                    if defined:
-                        weCare = (value != notOfInterest)
-                    elif defaultValue and len(notOfInterest):
-                        weCare = (defaultValue != notOfInterest)
-                    else:
-                        weCare = False
-
-                    if weCare and start >= 0:
-                        maskStart = start - self.lineOffset
-                        maskEnd = end - self.lineOffset - adjustment
-                        maskLength = maskEnd - maskStart
-                        attrMask[maskStart:maskEnd] = \
-                                                  [attrIndicator] * maskLength
-                if (end <= i):
+            offset = self.lineOffset
+            while offset < lineEndOffset:
+                attributes, startOffset, endOffset = \
+                            script.getTextAttributes(self.accessible,
+                                                     offset, True)
+                if endOffset <= offset:
                     break
+                mask = settings.TEXT_ATTR_BRAILLE_NONE
+                offset = endOffset
+                for attrib in attributes:
+                    if enabledAttributes.get(attrib, '') != '':
+                        if enabledAttributes[attrib] != attributes[attrib]:
+                            mask = attrIndicator
+                            break
+                if mask != settings.TEXT_ATTR_BRAILLE_NONE:
+                    maskStart = max(startOffset - self.lineOffset, 0)
+                    maskEnd = min(endOffset - self.lineOffset, stringLength)
+                    for i in xrange(maskStart, maskEnd):
+                        regionMask[i] |= attrIndicator
 
-                i = end
+        if selIndicator:
+            selections = script.getTextSelections(self.accessible)
+            for startOffset, endOffset in selections:
+                maskStart = max(startOffset - self.lineOffset, 0)
+                maskEnd = min(endOffset - self.lineOffset, stringLength)
+                for i in xrange(maskStart, maskEnd):
+                    regionMask[i] |= selIndicator
 
-        if indicateSel:
-            nSelections = text.getNSelections()
-            if nSelections:
-                for i in range(0, nSelections):
-                    [start, end] = text.getSelection(i)
-                    maskStart = max(self.lineOffset, start) - self.lineOffset
-                    maskEnd = min(end - self.lineOffset, stringLength)
-                    # Combine the selection indicator with the attribute
-                    # indicator.
-                    #
-                    for j in range(maskStart, maskEnd):
-                        if attrMask[j] != '\x00' \
-                           and attrMask[j] != selIndicator:
-                            attrMask[j] = '\xc0'
-                        else:
-                            attrMask[j] = selIndicator
-
-        mask = "".join(attrMask)
+        if self.contracted:
+            contractedMask = [0] * len(self.rawLine)
+            outPos = self.outPos[len(self.label):]
+            if self.label:
+                # Transform the offsets.
+                outPos = \
+                       [offset - len(self.label) - 1 for offset in outPos]
+            for i, m in enumerate(regionMask):
+                try:
+                    contractedMask[outPos[i]] |= m
+                except IndexError:
+                    continue
+            regionMask = contractedMask
 
         # Add empty mask characters for the EOL character as well as for
         # any label that might be present.
         #
-        for i in range (0, len(self.eol)):
-            mask += '\x00'
-        if self.label:
-            for i in range (0, len(self.label) + 1):
-                mask = '\x00' + mask
+        regionMask += [0]*len(self.eol)
 
-        return mask
+        if self.label:
+            regionMask = [0]*len(self.label) + regionMask
+
+        return ''.join(map(chr, regionMask))
+
+    def contractLine(self, line, cursorOffset=0, expandOnCursor=True):
+        contracted, inPos, outPos, cursorPos = Region.contractLine(
+            self, line, cursorOffset, expandOnCursor)
+        
+        return contracted + self.eol, inPos, outPos, cursorPos
+
+    def displayToBufferOffset(self, display_offset):
+        offset = Region.displayToBufferOffset(self, display_offset)
+        offset += self.startOffset
+        offset -= len(self.label)
+        return offset
+
+    def setContractedBraille(self, contracted):
+        Region.setContractedBraille(self, contracted)
+        if not contracted:
+            self.string += self.eol
 
 class ReviewComponent(Component):
     """A subclass of Component that is to be used for flat review mode."""
@@ -541,8 +652,8 @@ class ReviewComponent(Component):
                         for this Region if it gets focus.
         - zone: the flat review Zone associated with this component
         """
-
-        Component.__init__(self, accessible, string, cursorOffset)
+        Component.__init__(self, accessible, string,
+                           cursorOffset, expandOnCursor=True)
         self.zone = zone
 
 class ReviewText(Region):
@@ -561,8 +672,7 @@ class ReviewText(Region):
         - lineOffset: the character offset into where the text line starts
         - zone: the flat review Zone associated with this component
         """
-
-        Region.__init__(self, string)
+        Region.__init__(self, string, expandOnCursor=True)
         self.accessible = accessible
         self.lineOffset = lineOffset
         self.zone = zone
@@ -573,8 +683,9 @@ class ReviewText(Region):
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
+        offset = self.displayToBufferOffset(offset)
         newCaretOffset = self.lineOffset + offset
-        self.accessible.queryText().setCaretOffset(newCaretOffset)
+        orca_state.activeScript.setCaretOffset(self.accessible, newCaretOffset)
 
 class Line:
     """A horizontal line on the display.  Each Line is composed of a sequential
@@ -631,10 +742,11 @@ class Line:
         # for a region, and then pass the event off to the region for
         # handling.
         #
-        region = None
+        foundRegion = None
         string = ""
         pos = 0
         for region in self.regions:
+            foundRegion = region
             string = string + region.string
             if len(string.decode("UTF-8")) > offset:
                 break
@@ -644,7 +756,7 @@ class Line:
         if offset >= len(string.decode("UTF-8")):
             return [None, -1]
         else:
-            return [region, offset - pos]
+            return [foundRegion, offset - pos]
 
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
@@ -656,6 +768,10 @@ class Line:
         if region:
             region.processRoutingKey(regionOffset)
 
+    def setContractedBraille(self, contracted):
+        for region in self.regions:
+            region.setContractedBraille(contracted)
+
 def getRegionAtCell(cell):
     """Given a 1-based cell offset, return the braille region
     associated with that cell in the form of [region, offsetinregion]
@@ -664,8 +780,8 @@ def getRegionAtCell(cell):
     in the region, where 0 represents the beginning of the region, """
 
     if len(_lines) > 0:
-        offset = (cell - 1) + _viewport[0]
-        lineNum = _viewport[1]
+        offset = (cell - 1) + viewport[0]
+        lineNum = viewport[1]
         return _lines[lineNum].getRegionAtOffset(offset)
     else:
         return [None, -1]
@@ -677,11 +793,11 @@ def clear():
 
     global _lines
     global _regionWithFocus
-    global _viewport
+    global viewport
 
     _lines = []
     _regionWithFocus = None
-    _viewport = [0, 0]
+    viewport = [0, 0]
 
 def setLines(lines):
     global _lines
@@ -697,15 +813,13 @@ def addLine(line):
     - line: an instance of Line to add.
     """
 
-    global _lines
-
     _lines.append(line)
     line._index = len(_lines)
 
 def getShowingLine():
     """Returns the Line that is currently being painted on the display.
     """
-    return _lines[_viewport[1]]
+    return _lines[viewport[1]]
 
 def setFocus(region, panToFocus=True):
     """Specififes the region with focus.  This region will be positioned
@@ -735,7 +849,7 @@ def setFocus(region, panToFocus=True):
     for line in _lines:
         for reg in line.regions:
             if reg == _regionWithFocus:
-                _viewport[1] = lineNum
+                viewport[1] = lineNum
                 done = True
                 break
         if done:
@@ -743,7 +857,7 @@ def setFocus(region, panToFocus=True):
         else:
             lineNum += 1
 
-    line = _lines[_viewport[1]]
+    line = _lines[viewport[1]]
     [string, offset, attributeMask] = line.getLineInfo()
 
     # If the cursor is too far right, we scroll the viewport
@@ -752,7 +866,7 @@ def setFocus(region, panToFocus=True):
     if _regionWithFocus.cursorOffset >= _displaySize[0]:
         offset += _regionWithFocus.cursorOffset - _displaySize[0] + 1
 
-    _viewport[0] = max(0, offset)
+    viewport[0] = max(0, offset)
 
 def refresh(panToCursor=True, targetCursorCell=0):
     """Repaints the Braille on the physical display.  This clips the entire
@@ -798,7 +912,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     # Now, we figure out the 0-based offset for where the cursor
     # actually is in the string.
     #
-    line = _lines[_viewport[1]]
+    line = _lines[viewport[1]]
     [string, focusOffset, attributeMask] = line.getLineInfo()
     cursorOffset = -1
     if focusOffset >= 0:
@@ -811,15 +925,15 @@ def refresh(panToCursor=True, targetCursorCell=0):
     #
     if panToCursor and (cursorOffset >= 0):
         if len(string.decode("UTF-8")) <= _displaySize[0]:
-            _viewport[0] = 0
+            viewport[0] = 0
         elif targetCursorCell:
-            _viewport[0] = max(0, cursorOffset - targetCursorCell + 1)
-        elif cursorOffset < _viewport[0]:
-            _viewport[0] = max(0, cursorOffset)
-        elif cursorOffset >= (_viewport[0] + _displaySize[0]):
-            _viewport[0] = max(0, cursorOffset - _displaySize[0] + 1)
+            viewport[0] = max(0, cursorOffset - targetCursorCell + 1)
+        elif cursorOffset < viewport[0]:
+            viewport[0] = max(0, cursorOffset)
+        elif cursorOffset >= (viewport[0] + _displaySize[0]):
+            viewport[0] = max(0, cursorOffset - _displaySize[0] + 1)
 
-    startPos = _viewport[0]
+    startPos = viewport[0]
     endPos = startPos + _displaySize[0]
 
     # Now normalize the cursor position to BrlTTY, which uses 1 as
@@ -843,14 +957,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     substring = string[startPos:endPos].encode("UTF-8")
     if useBrlAPIBindings:
         if brlAPIRunning:
-            try:
-                # The name after (and including) BrlTTY v3.8 revision 2810
-                #
-                writeStruct = brlapi.WriteStruct()
-            except:
-                # The name before BrlTTY v3.8 revision 2810
-                #
-                writeStruct = brlapi.Write()
+            writeStruct = brlapi.WriteStruct()
             writeStruct.regionBegin = 1
             writeStruct.regionSize = len(substring.decode("UTF-8"))
             while writeStruct.regionSize < _displaySize[0]:
@@ -895,6 +1002,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
         monitor.writeText(cursorCell, substring, subMask)
     elif monitor:
         monitor.destroy()
+        monitor = None
 
     beginningIsShowing = startPos == 0
     endIsShowing = endPos >= len(string)
@@ -947,15 +1055,15 @@ def panLeft(panAmount=0):
     Returns True if a pan actually happened.
     """
 
-    oldX = _viewport[0]
+    oldX = viewport[0]
 
     if panAmount == 0:
         panAmount = _displaySize[0]
 
-    if _viewport[0] > 0:
-        _viewport[0] = max(0, _viewport[0] - panAmount)
+    if viewport[0] > 0:
+        viewport[0] = max(0, viewport[0] - panAmount)
 
-    return oldX != _viewport[0]
+    return oldX != viewport[0]
 
 def panRight(panAmount=0):
     """Pans the display to the right, limiting the pan to the length
@@ -968,31 +1076,31 @@ def panRight(panAmount=0):
     Returns True if a pan actually happened.
     """
 
-    oldX = _viewport[0]
+    oldX = viewport[0]
 
     if panAmount == 0:
         panAmount = _displaySize[0]
 
     if len(_lines) > 0:
-        lineNum = _viewport[1]
-        newX = _viewport[0] + panAmount
+        lineNum = viewport[1]
+        newX = viewport[0] + panAmount
         [string, focusOffset, attributeMask] = _lines[lineNum].getLineInfo()
         if newX < len(string.decode("UTF-8")):
-            _viewport[0] = newX
+            viewport[0] = newX
 
-    return oldX != _viewport[0]
+    return oldX != viewport[0]
 
 def panToOffset(offset):
     """Automatically pan left or right to make sure the current offset is
     showing."""
 
-    while offset < _viewport[0]:
+    while offset < viewport[0]:
         debug.println(debug.LEVEL_FINEST,
                       "braille.panToOffset (left) %d" % offset)
         if not panLeft():
             break
 
-    while offset >= (_viewport[0] + _displaySize[0]):
+    while offset >= (viewport[0] + _displaySize[0]):
         debug.println(debug.LEVEL_FINEST,
                       "braille.panToOffset (right) %d" % offset)
         if not panRight():
@@ -1054,11 +1162,17 @@ def _processBrailleEvent(command):
 
     if (command >= 0x100) and (command < (0x100 + _displaySize[0])):
         if len(_lines) > 0:
-            cursor = (command - 0x100) + _viewport[0]
-            lineNum = _viewport[1]
+            cursor = (command - 0x100) + viewport[0]
+            lineNum = viewport[1]
             _lines[lineNum].processRoutingKey(cursor)
             consumed = True
 
+    if command in (0x2141, 65): # Toggle six dot braille
+        settings.enableContractedBraille = not settings.enableContractedBraille
+        for line in _lines:
+            line.setContractedBraille(settings.enableContractedBraille)
+        refresh()
+
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
 
@@ -1070,11 +1184,11 @@ def _brlAPIKeyReader(source, condition):
     """
     key = brlAPI.readKey(False)
     if key:
-        flags = key >> 32
+        #flags = key >> 32
         lower = key & 0xFFFFFFFF
-        keyType = lower >> 29
+        #keyType = lower >> 29
         keyCode = lower & 0x1FFFFFFF
-
+        
         # [[TODO: WDW - HACK If we have a cursor routing key, map
         # it back to the code we used to get with earlier versions
         # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
@@ -1110,9 +1224,11 @@ def setupKeyRanges(keys):
         #
         for key in keys:
             keySet.append(brlapi.KEY_TYPE_CMD | key)
-
+            
         brlAPI.acceptKeys(brlapi.rangeType_command, keySet)
 
+        brlAPI.acceptKeys(brlapi.rangeType_key, [65])
+
         debug.println(debug.LEVEL_FINEST, "Using BrlAPI v0.5.0+")
     except:
         debug.printException(debug.LEVEL_FINEST)
