@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__  = "$Id: focus_tracking_presenter.py 3963 2008-06-05 18:10:20Z wwalker $"
-__version__   = "$Revision: 3963 $"
-__date__      = "$Date: 2008-06-05 14:10:20 -0400 (Thu, 05 Jun 2008) $"
+__id__  = "$Id: focus_tracking_presenter.py 3782 2008-03-28 20:41:31Z eitani $"
+__version__   = "$Revision: 3782 $"
+__date__      = "$Date: 2008-03-28 16:41:31 -0400 (Fri, 28 Mar 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -266,7 +266,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         return script
 
-    def _getScript(self, app):
+    def getScript(self, app):
         """Get a script for an app (and make it if necessary).  This is used
         instead of a simple calls to Script's constructor.
 
@@ -575,7 +575,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # displayed. See Orca bug #409731 for more details.
             #
             if not event.type.startswith("mouse:"):
-                s = self._getScript(event.host_application or \
+                s = self.getScript(event.host_application or \
                                       event.source.getApplication())
             else:
                 s = orca_state.activeScript
@@ -610,7 +610,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                       + orca_state.activeScript.name)
 
                         self.setActiveScript(
-                            self._getScript(event.host_application or \
+                            self.getScript(event.host_application or \
                                               event.source.getApplication()))
 
                         # Load in the application specific settings for the
@@ -658,8 +658,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - e: an at-spi event.
         """
 
-        asyncMode = settings.asyncMode
-
         if settings.debugEventQueue:
             if self._enqueueEventCount:
                 debug.println(debug.LEVEL_ALL,
@@ -732,16 +730,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "---------> QUEUEING EVENT %s" % e.type)
 
-            # Some toolkits (e.g., Java - see bug #531869) need to
-            # have their events processed immediately.
-            #
-            try:
-                if event.host_application.toolkitName \
-                    in settings.synchronousToolkits:
-                    asyncMode = False
-            except:
-                pass
-
         if event:
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
@@ -758,7 +746,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...put complete")
-            if asyncMode and (not self._gidleId):
+            if settings.asyncMode and (not self._gidleId):
                 if settings.gilSleepTime:
                     time.sleep(settings.gilSleepTime)
                 self._gidleId = gobject.idle_add(self._dequeueEvent)
@@ -771,7 +759,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "           ...released")
 
-            if not asyncMode:
+            if not settings.asyncMode:
                 self._dequeueEvent()
 
         if settings.debugEventQueue:
@@ -973,7 +961,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         """Restores script and application state information."""
         try:
             for [app, appState] in self._appStateInfo:
-                script = self._getScript(app)
+                script = self.getScript(app)
                 script.setAppState(appState)
         except:
             pass
@@ -991,7 +979,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self._restoreAppStates()
 
-        self.setActiveScript(self._getScript(None))
+        self.setActiveScript(self.getScript(None))
 
         # Tell BrlTTY which commands we care about.
         #
