@@ -23,13 +23,13 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py 2437 2007-05-31 12:41:26Z wwalker $"
-__version__   = "$Revision: 2437 $"
-__date__      = "$Date: 2007-05-31 08:41:26 -0400 (Thu, 31 May 2007) $"
+__id__        = "$Id: input_event.py 3599 2008-02-19 01:22:55Z joanied $"
+__version__   = "$Revision: 3599 $"
+__date__      = "$Date: 2008-02-18 20:22:55 -0500 (Mon, 18 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import atspi
+import pyatspi
 import debug
 import settings
 import time
@@ -42,15 +42,15 @@ SPEECH_EVENT       = "speech"
 
 class InputEvent:
 
-    def __init__(self, type):
+    def __init__(self, eventType):
         """Creates a new input event of the given type.
 
         Arguments:
-        - type: the input event type (one of KEYBOARD_EVENT, BRAILLE_EVENT,
+        - eventType: the input event type (one of KEYBOARD_EVENT, BRAILLE_EVENT,
                 MOUSE_BUTTON_EVENT, MOUSE_MOTION_EVENT, SPEECH_EVENT).
         """
 
-        self.type = type
+        self.type = eventType
 
 class KeyboardEvent(InputEvent):
 
@@ -70,12 +70,24 @@ class KeyboardEvent(InputEvent):
         # mapping ASCII control characters to UTF-8.]]]
         #
         event_string = event.event_string
-        if (event.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL)) \
+        if (event.modifiers & (1 << pyatspi.MODIFIER_CONTROL)) \
             and (not event.is_text) and (len(event_string) == 1):
             value = ord(event.event_string[0])
             if value < 32:
                 event_string = chr(value + 0x40)
 
+        # Filter out the NUMLOCK modifier -- it always causes problems.
+        #
+        mask = (1 << settings.MODIFIER_ORCA |
+                1 << pyatspi.MODIFIER_SHIFT |
+                1 << pyatspi.MODIFIER_SHIFTLOCK |
+                1 << pyatspi.MODIFIER_CONTROL |
+                1 << pyatspi.MODIFIER_ALT |
+                1 << pyatspi.MODIFIER_META |
+                1 << pyatspi.MODIFIER_META2 |
+                1 << pyatspi.MODIFIER_META3)
+        event.modifiers = event.modifiers & mask
+
         InputEvent.__init__(self, KEYBOARD_EVENT)
         self.type = event.type
         self.hw_code = event.hw_code
@@ -142,13 +154,13 @@ class InputEventHandler:
                             the function will be called no matter what.
         """
 
-        self._function = function
-        self._description = description
+        self.function = function
+        self.description = description
         self._learnModeEnabled = learnModeEnabled
 
     def __eq__(self, other):
         """Compares one input handler to another."""
-        return (self._function == other._function)
+        return (self.function == other.function)
 
     def processInputEvent(self, script, inputEvent):
         """Processes an input event.  If settings.learnModeEnabled is True,
@@ -169,18 +181,26 @@ class InputEventHandler:
         consumed = False
 
         if settings.learnModeEnabled and self._learnModeEnabled:
-            if self._description:
+            if self.description:
                 # These imports are here to eliminate circular imports.
                 #
                 import braille
                 import speech
-                braille.displayMessage(self._description)
-                speech.speak(self._description)
+                braille.displayMessage(self.description)
+                speech.speak(self.description)
                 consumed = True
         else:
             try:
-                consumed = self._function(script, inputEvent)
+                consumed = self.function(script, inputEvent)
             except:
                 debug.printException(debug.LEVEL_SEVERE)
 
         return consumed
+
+def keyEventToString(event):
+    return ("KEYEVENT: type=%d\n" % event.type) \
+        + ("          hw_code=%d\n" % event.hw_code) \
+        + ("          modifiers=%d\n" % event.modifiers) \
+        + ("          event_string=(%s)\n" % event.event_string) \
+        + ("          is_text=%s\n" % event.is_text) \
+        + ("          time=%f" % time.time())
