@@ -20,9 +20,9 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 3426 2008-01-02 19:51:30Z joanied $"
-__version__   = "$Revision: 3426 $"
-__date__      = "$Date: 2008-01-02 14:51:30 -0500 (Wed, 02 Jan 2008) $"
+__id__        = "$Id: mag.py 3824 2008-04-08 22:29:11Z joanied $"
+__version__   = "$Revision: 3824 $"
+__date__      = "$Date: 2008-04-08 18:29:11 -0400 (Tue, 08 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -885,9 +885,13 @@ def magnifyAccessible(event, obj, extents=None):
         try:
             text = obj.queryText()
             if text and (text.caretOffset >= 0):
+                offset = text.caretOffset
+                if offset == text.characterCount:
+                    offset -= 1
                 [x, y, width, height] = \
-                    text.getCharacterExtents(text.caretOffset, 0)
-                haveSomethingToMagnify = True
+                    text.getCharacterExtents(offset, 0)
+                haveSomethingToMagnify = (width + height > 0)
+                    
         except:
             haveSomethingToMagnify = False
 
