@@ -27,9 +27,9 @@ The following script specific key sequences are supported:
   Insert-[1-9]  -  Speak and braille a previous chat room message.
 """
 
-__id__        = "$Id: gaim.py 3839 2008-04-19 17:13:42Z richb $"
-__version__   = "$Revision: 3839 $"
-__date__      = "$Date: 2008-04-19 13:13:42 -0400 (Sat, 19 Apr 2008) $"
+__id__        = "$Id: gaim.py 3838 2008-04-19 17:13:25Z richb $"
+__version__   = "$Revision: 3838 $"
+__date__      = "$Date: 2008-04-19 13:13:25 -0400 (Sat, 19 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -365,6 +365,7 @@ class Script(default.Script):
         self.focusedChannelRadioButton = None
         self.allChannelsRadioButton = None
         self.allMessagesRadioButton = None
+        self.buddyTypingCheckButton = None
 
         default.Script.__init__(self, app)
 
@@ -394,6 +395,11 @@ class Script(default.Script):
                 _("Toggle whether we prefix chat room messages with " \
                   "the name of the chat room."))
 
+        self.inputEventHandlers["toggleBuddyTypingHandler"] = \
+            input_event.InputEventHandler(
+                Script.toggleBuddyTyping,
+                _("Toggle whether we announce when our buddies are typing."))
+
         # Add the chat room message history event handler.
         #
         self.inputEventHandlers["reviewMessage"] = \
@@ -419,6 +425,13 @@ class Script(default.Script):
                 1 << settings.MODIFIER_ORCA,
                 self.inputEventHandlers["togglePrefixHandler"]))
 
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "",
+                None,
+                None,
+                self.inputEventHandlers["toggleBuddyTypingHandler"]))
+
         # keybindings to provide chat room message history.
         #
         messageKeys = [ "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9" ]
@@ -463,6 +476,16 @@ class Script(default.Script):
         gtk.ToggleButton.set_active(self.speakNameCheckButton,
                                     prefixChatMessage)
 
+        # Translators: If this checkbox is checked, then Orca will tell
+        # you when one of your buddies is typing a message.
+        #
+        label = _("Announce when your _buddies are typing")
+        self.buddyTypingCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.buddyTypingCheckButton)
+        gtk.Box.pack_start(vbox, self.buddyTypingCheckButton, False, False, 0)
+        gtk.ToggleButton.set_active(self.buddyTypingCheckButton,
+                                    announceBuddyTyping)
+
         # "Speak Messages" frame.
         #
         messagesFrame = gtk.Frame()
@@ -534,13 +557,17 @@ class Script(default.Script):
         - prefs: file handle for application preferences.
         """
 
-        global prefixChatMessage, speakMessages
+        global announceBuddyTyping, prefixChatMessage, speakMessages
 
         prefixChatMessage = self.speakNameCheckButton.get_active()
         prefs.writelines("\n")
         prefs.writelines("orca.scripts.gaim.prefixChatMessage = %s\n" % \
                          prefixChatMessage)
 
+        announceBuddyTyping = self.buddyTypingCheckButton.get_active()
+        prefs.writelines("orca.scripts.gaim.announceBuddyTyping = %s\n" % \
+                         announceBuddyTyping)
+
         if self.allMessagesRadioButton.get_active():
             speakMessages = SPEAK_ALL_MESSAGES
             option = "orca.scripts.gaim.SPEAK_ALL_MESSAGES"
@@ -598,6 +625,26 @@ class Script(default.Script):
 
         return True
 
+    def toggleBuddyTyping(self, inputEvent):
+        """ Toggle whether we announce when our buddies are typing a message.
+
+        Arguments:
+        - inputEvent: if not None, the input event that caused this action.
+        """
+
+        global announceBuddyTyping
+
+        debug.println(self.debugLevel, "gaim.toggleBuddyTyping.")
+
+        line = _("announce when your buddies are typing.")
+        announceBuddyTyping = not announceBuddyTyping
+        if not announceBuddyTyping:
+            line = _("Do not announce when your buddies are typing.")
+
+        speech.speak(line)
+
+        return True
+
     def utterMessage(self, chatRoomName, message, hasFocus=True):
         """ Speak/braille a chat room message.
 
@@ -704,53 +751,6 @@ class Script(default.Script):
                         line = _("New chat tab %s") % child.name
                         speech.speak(line)
 
-    def attribsToDictionary(self, dict_string):
-        """Creates a Python dict from a typical attributes list returned from
-        different AT-SPI methods.
-
-
-        Arguments:
-        - dict_string: A list of colon seperated key/value pairs seperated by
-        semicolons.
-        Returns a Python dict of the given attributes.
-        """
-        try:
-            return dict(
-                map(lambda pair: pair.strip().split(':'),
-                    dict_string.strip(';').split(';')))
-        except ValueError:
-            return {}
-
-    def getTextAttributes(self, acc, offset, get_defaults=False):
-        """Get the text attributes run for a given offset in a given accessible
-
-        Arguments:
-        - acc: An accessible.
-        - offset: Offset in the accessible's text for which to retrieve the
-        attributes.
-        - get_defaults: Get the default attributes as well as the unique ones.
-        Default is True
-
-        Returns a dictionary of attributes, a start offset where the attributes
-        begin, and an end offset. Returns ({}, 0, 0) if the accessible does not
-        support the text attribute.
-        """
-
-        rv = {}
-        try:
-            texti = acc.queryText()
-        except:
-            return rv, 0, 0
-
-        if get_defaults:
-            rv.update(self.attribsToDictionary(texti.getDefaultAttributes()))
-
-        attrib_str, start, end = texti.getAttributes(offset)
-
-        rv.update(self.attribsToDictionary(attrib_str))
-
-        return rv, start, end
-
     def isBuddyListEvent(self, event):
         """If pidgin gets a status changed message for one of the users
         buddies then just ignore it. See bug #525644 for more details.
