@@ -21,9 +21,9 @@
 of manipulating text - both the user and the system put text into the
 system and we try to determine which was which and why."""
 
-__id__        = "$Id: gnome-terminal.py 3649 2008-02-27 22:56:36Z richb $"
-__version__   = "$Revision: 3649 $"
-__date__      = "$Date: 2008-02-27 17:56:36 -0500 (Wed, 27 Feb 2008) $"
+__id__        = "$Id: gnome-terminal.py 3836 2008-04-19 17:07:46Z richb $"
+__version__   = "$Revision: 3836 $"
+__date__      = "$Date: 2008-04-19 13:07:46 -0400 (Sat, 19 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -93,7 +93,8 @@ class Script(default.Script):
         # and if the old locus of focus also had a "terminal role.
         # See bug #518762 for more details.
         #
-        if newLocusOfFocus.getRole() == pyatspi.ROLE_TERMINAL:
+        if newLocusOfFocus and \
+           newLocusOfFocus.getRole() == pyatspi.ROLE_TERMINAL:
             pageTab = event.source.parent.parent.parent
             if oldLocusOfFocus.getRole() == pyatspi.ROLE_TERMINAL and \
                pageTab.getRole() == pyatspi.ROLE_PAGE_TAB and \
@@ -266,3 +267,33 @@ class Script(default.Script):
            and self.isWordDelimiter(text.decode("UTF-8")[-1:]):
             if matchFound:
                 self.echoPreviousWord(event.source)
+
+    def getTextLineAtCaret(self, acc):
+        """Gets the line of text where the caret is.
+
+        Argument:
+        - obj: an Accessible object that implements the AccessibleText
+               interface
+
+        Returns the [string, caretOffset, startOffset] for the line of text
+        where the caret is.
+        """
+        string, caretOffset, lineOffset = \
+                default.Script.getTextLineAtCaret(self, acc)
+
+        # Sometimes, gnome-terminal will give us very odd values when
+        # the user is editing using 'vi' and has positioned the caret
+        # at the first character of the first line.  In this case, we
+        # end up getting a very large negative number for the line offset.
+        # So, we just assume the user is at the first character.
+        #
+        if lineOffset < 0:
+            caretOffset = 0
+            lineOffset = 0
+            texti = acc.queryText()
+            string, startOffset, endOffset = \
+                    texti.getTextAtOffset(0,
+                                          pyatspi.TEXT_BOUNDARY_LINE_START)
+
+        return string, caretOffset, lineOffset
+        
