@@ -19,13 +19,12 @@
 
 """Custom script for planner."""
 
-__id__        = "$Id: planner.py 3560 2008-02-11 17:35:26Z wwalker $"
-__version__   = "$Revision: 3560 $"
-__date__      = "$Date: 2008-02-11 12:35:26 -0500 (Mon, 11 Feb 2008) $"
+__id__        = "$Id: planner.py 3726 2008-03-19 00:15:13Z wwalker $"
+__version__   = "$Revision: 3726 $"
+__date__      = "$Date: 2008-03-18 20:15:13 -0400 (Tue, 18 Mar 2008) $"
 __copyright__ = "Copyright (c) 2006-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import orca.debug as debug
 import orca.default as default
 import orca.braille as braille
 import orca.settings as settings
@@ -74,26 +73,6 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         if self._script.isDesiredFocusedItem(obj, roleList) and not obj.name:
             text += _("Display more options")
 
-        # Second special case is each one of the four graphics toggle
-        # buttons in the main window Application should implement an
-        # accessible relationship between the button and the label,
-        # but until this is made we append for each one the button the
-        # label that should be associated and its state (checked or
-        # not)
-        #
-        rolesList = [pyatspi.ROLE_TOGGLE_BUTTON, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_FILLER, \
-                     pyatspi.ROLE_PANEL, \
-                     pyatspi.ROLE_PANEL]
-        if self._script.isDesiredFocusedItem(obj, rolesList):
-            debug.println(debug.LEVEL_FINEST,
-                          "planner.onFocus - main window: " \
-                          + "one of the four graphic toggle buttons.")
-            filler = obj.parent
-            allLabels = self._script.findByRole(filler, pyatspi.ROLE_LABEL)
-            text += allLabels[0].name
-
         text = self._script.appendString(text, self._getTextForRole(obj))
 
         if obj.getState().contains(pyatspi.STATE_CHECKED):
@@ -102,10 +81,9 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
             brailleindicatorindex = 0
 
         regions = []
-        componentRegion = braille.Component(
-            obj, text,
-            indicator=\
-            settings.brailleRadioButtonIndicators[brailleindicatorindex])
+        indicator = \
+            settings.brailleRadioButtonIndicators[brailleindicatorindex]
+        componentRegion = braille.Component(obj, text, indicator=indicator)
         regions.append(componentRegion)
 
         return [regions, componentRegion]
@@ -154,41 +132,6 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
             return utterances
 
-        # Application should implement an accessible relationship
-        # between the button and the label, but until this is made we
-        # append for each one the button the label that should be
-        # associated and its state (checked or not)
-        #
-        roleList = [pyatspi.ROLE_TOGGLE_BUTTON, \
-                    pyatspi.ROLE_FILLER, \
-                    pyatspi.ROLE_FILLER, \
-                    pyatspi.ROLE_PANEL, \
-                    pyatspi.ROLE_PANEL]
-
-        if self._script.isDesiredFocusedItem(obj, roleList):
-            debug.println(debug.LEVEL_FINEST,
-                          "planner.onFocus - main window: " \
-                          + "one of the four graphic toggle buttons.")
-            if not already_focused:
-                filler = obj.parent
-                allLabels = self._script.findByRole(filler, pyatspi.ROLE_LABEL)
-                tmp.append(allLabels[0].name)
-                tmp.extend(self._getDefaultSpeech(obj, already_focused))
-                if obj.getState().contains(pyatspi.STATE_CHECKED):
-                    tmp.append(_("pressed"))
-                else:
-                    tmp.append(_("not pressed"))
-
-                utterances.extend(tmp)
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                if obj.getState().contains(pyatspi.STATE_CHECKED):
-                    utterances.append(_("pressed"))
-                else:
-                    utterances.append(_("not pressed"))
-
-            return utterances
-
         return self._getSpeechForLabel(obj, already_focused)
 
 ########################################################################
