@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -21,10 +21,10 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 3661 2008-02-29 18:18:36Z shaeger $"
-__version__   = "$Revision: 3661 $"
-__date__      = "$Date: 2008-02-29 13:18:36 -0500 (Fri, 29 Feb 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: settings.py 3819 2008-04-07 19:20:05Z wwalker $"
+__version__   = "$Revision: 3819 $"
+__date__      = "$Date: 2008-04-07 15:20:05 -0400 (Mon, 07 Apr 2008) $"
+__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
@@ -157,7 +157,9 @@ userCustomizableSettings = [
     "enableProgressBarUpdates",
     "progressBarUpdateInterval",
     "enableContractedBraille",
-    "brailleContractionTable"
+    "brailleContractionTable",
+    "enableMouseReview",
+    "mouseDwellDelay"
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -221,10 +223,10 @@ brailleRolenameStyle    = BRAILLE_ROLENAME_STYLE_LONG
 # The values represent the character to be used in the attrOr
 # field of brlAPI's writeStruct.
 #
-BRAILLE_SEL_NONE = '\x00' # 00000000
-BRAILLE_SEL_7    = '\x40' # 01000000
-BRAILLE_SEL_8    = '\x80' # 10000000
-BRAILLE_SEL_BOTH = '\xc0' # 11000000
+BRAILLE_SEL_NONE = 0x00 # 00000000
+BRAILLE_SEL_7    = 0x40 # 01000000
+BRAILLE_SEL_8    = 0x80 # 10000000
+BRAILLE_SEL_BOTH = 0xc0 # 11000000
 brailleSelectorIndicator = BRAILLE_SEL_BOTH
 
 # Speech punctuation levels (see verbalizePunctuationStyle).
@@ -256,7 +258,7 @@ speechPitchDelta        = 0.5
 # The port to listen on if orca is to act as an HTTP server
 # (mainly as a speech server for self-voicing applications).
 #
-httpServerPort          = 20433
+httpServerPort          = 0
 
 # The number of attempts to retry setting up an HTTP server
 # connection (each time incrementing the port number by 1).
@@ -593,10 +595,10 @@ enabledBrailledTextAttributes = \
 # The values represent the character to be used in the attrOr
 # field of brlAPI's writeStruct.
 #
-TEXT_ATTR_BRAILLE_NONE = '\x00' # 00000000
-TEXT_ATTR_BRAILLE_7    = '\x40' # 01000000
-TEXT_ATTR_BRAILLE_8    = '\x80' # 10000000
-TEXT_ATTR_BRAILLE_BOTH = '\xc0' # 11000000
+TEXT_ATTR_BRAILLE_NONE = 0x00 # 00000000
+TEXT_ATTR_BRAILLE_7    = 0x40 # 01000000
+TEXT_ATTR_BRAILLE_8    = 0x80 # 10000000
+TEXT_ATTR_BRAILLE_BOTH = 0xc0 # 11000000
 textAttributesBrailleIndicator = TEXT_ATTR_BRAILLE_NONE
 
 # The limit to enable a repeat character count to be spoken.
@@ -919,3 +921,17 @@ brailleContractionTable = ''
 # Use Collection Interface?
 # 
 useCollection = True
+
+
+# Report object under mouse.
+#
+enableMouseReview = False
+
+# Mouse dwell delay in milliseconds for mouse review mode. 
+# If the value is zero, the review will be read time.
+#
+mouseDwellDelay = 0
+
+# Maximum allowed drift while pointer is dwelling in mouse review mode.
+# 
+mouseDwellMaxDrift = 3
