@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 3378 2007-12-14 20:20:12Z wwalker $"
-__version__   = "$Revision: 3378 $"
-__date__      = "$Date: 2007-12-14 15:20:12 -0500 (Fri, 14 Dec 2007) $"
+__id__        = "$Id: where_am_I.py 3834 2008-04-17 18:22:54Z joanied $"
+__version__   = "$Revision: 3834 $"
+__date__      = "$Date: 2008-04-17 14:22:54 -0400 (Thu, 17 Apr 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -50,7 +50,7 @@ class WhereAmI:
         self._defaultButton = None
         self._lastAttributeString = ""
 
-    def whereAmI(self, obj, doubleClick, titleOrStatus):
+    def whereAmI(self, obj, basicOnly):
         """Speaks information about the current object of interest, including
         the object itself, which window it is in, which application, which
         workspace, etc.
@@ -74,8 +74,7 @@ class WhereAmI:
            \n  parent label= %s \
            \n  parent name=%s \
            \n  parent role=%s \
-           \n  double-click=%s \
-           \n  titleOrStatus=%s" % \
+           \n  basicOnly=%s" % \
             (self._getObjLabel(obj),
              self._getObjName(obj),
              obj.getRoleName(),
@@ -83,78 +82,81 @@ class WhereAmI:
              self._getObjLabel(obj.parent),
              self._getObjName(obj.parent),
              obj.parent.getRoleName(),
-             doubleClick,
-             titleOrStatus))
+             basicOnly))
 
         role = obj.getRole()
 
-        if titleOrStatus:
-            self._speakTitleOrStatus(obj, doubleClick)
-            return True
-
-        else:
-            toolbar = self._getToolbar(obj)
-            if toolbar:
-                self._speakToolbar(toolbar)
+        toolbar = self._getToolbar(obj)
+        if toolbar:
+            self._speakToolbar(toolbar)
 
         if role == pyatspi.ROLE_CHECK_BOX:
-            self._speakCheckBox(obj, doubleClick)
+            self._speakCheckBox(obj, basicOnly)
 
         elif role == pyatspi.ROLE_RADIO_BUTTON:
-            self._speakRadioButton(obj, doubleClick)
+            self._speakRadioButton(obj, basicOnly)
 
         elif role == pyatspi.ROLE_COMBO_BOX:
-            self._speakComboBox(obj, doubleClick)
+            self._speakComboBox(obj, basicOnly)
 
         elif role == pyatspi.ROLE_SPIN_BUTTON:
-            self._speakSpinButton(obj, doubleClick)
+            self._speakSpinButton(obj, basicOnly)
 
         elif role == pyatspi.ROLE_PUSH_BUTTON:
-            self._speakPushButton(obj, doubleClick)
+            self._speakPushButton(obj, basicOnly)
 
         elif role == pyatspi.ROLE_SLIDER:
-            self._speakSlider(obj, doubleClick)
+            self._speakSlider(obj, basicOnly)
 
         elif role in [pyatspi.ROLE_MENU,
                       pyatspi.ROLE_MENU_ITEM,
                       pyatspi.ROLE_CHECK_MENU_ITEM,
                       pyatspi.ROLE_RADIO_MENU_ITEM]:
-            self._speakMenuItem(obj, doubleClick)
+            self._speakMenuItem(obj, basicOnly)
 
         elif role == pyatspi.ROLE_PAGE_TAB:
-            self._speakPageTab(obj, doubleClick)
+            self._speakPageTab(obj, basicOnly)
 
         elif role in [pyatspi.ROLE_TEXT,
                       pyatspi.ROLE_TERMINAL]:
-            self._speakText(obj, doubleClick)
+            self._speakText(obj, basicOnly)
 
         elif role == pyatspi.ROLE_TABLE_CELL:
-            self._speakTableCell(obj, doubleClick)
+            self._speakTableCell(obj, basicOnly)
 
         elif role == pyatspi.ROLE_LIST_ITEM:
-            self._speakListItem(obj, doubleClick)
+            self._speakListItem(obj, basicOnly)
 
         elif role == pyatspi.ROLE_PARAGRAPH:
-            self._speakParagraph(obj, doubleClick)
+            self._speakParagraph(obj, basicOnly)
 
         elif role == pyatspi.ROLE_ICON:
-            self._speakIconPanel(obj, doubleClick)
+            self._speakIconPanel(obj, basicOnly)
             
         elif role == pyatspi.ROLE_LINK:
-            self._speakLink(obj, doubleClick)
+            self._speakLink(obj, basicOnly)
 
         elif role == pyatspi.ROLE_TOGGLE_BUTTON:
-            self._speakToggleButton(obj, doubleClick)
+            self._speakToggleButton(obj, basicOnly)
+
+        elif role == pyatspi.ROLE_SPLIT_PANE:
+            self._speakSplitPane(obj, basicOnly)
+
+        elif role == pyatspi.ROLE_LABEL:
+            self._speakLabel(obj, basicOnly)
+
+        elif role  == pyatspi.ROLE_LAYERED_PANE:
+            self._speakLayeredPane(obj, basicOnly)
 
         else:
-            self._speakGenericObject(obj, doubleClick)
+            self._speakGenericObject(obj, basicOnly)
 
-        if not doubleClick:
+        if basicOnly:
             self._speakObjDescription(obj)
 
         return True
 
-    def _speakCheckBox(self, obj, doubleClick):
+    def _speakCheckBox(self, obj, basicOnly):
         """Checkboxes present the following information
         (an example is 'Enable speech, checkbox checked, Alt E'):
         1. label
@@ -176,7 +178,7 @@ class WhereAmI:
                       % utterances)
         speech.speakUtterances(utterances)
 
-    def _speakRadioButton(self, obj, doubleClick):
+    def _speakRadioButton(self, obj, basicOnly):
         """Radio Buttons present the following information (an example is
         'Punctuation Level, Some, Radio button, selected, item 2 of 4, Alt M'):
 
@@ -224,7 +226,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakComboBox(self, obj, doubleClick):
+    def _speakComboBox(self, obj, basicOnly):
         """Comboboxes present the following information (an example is
         'Speech system: combo box, GNOME Speech Services, item 1 of 1,
         Alt S'):
@@ -256,14 +258,15 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakSpinButton(self, obj, doubleClick):
+    def _speakSpinButton(self, obj, basicOnly):
         """Spin Buttons present the following information (an example is
         'Scale factor: spin button, 4.00, Alt F'):
 
         1. label
         2. role
         3. current value
-        4. accelerator (i.e. Alt plus the underlined letter), if any
+        4. selected (if True).
+        5. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
@@ -276,6 +279,8 @@ class WhereAmI:
         name = self._getObjName(obj)
         utterances.append(name)
         
+        utterances.extend(self._getSpeechForAllTextSelection(obj))
+
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
@@ -283,7 +288,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakPushButton(self, obj, doubleClick):
+    def _speakPushButton(self, obj, basicOnly):
         """ Push Buttons present the following information (an example is
         'Apply button, Alt A'):
 
@@ -306,7 +311,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakSlider(self, obj, doubleClick):
+    def _speakSlider(self, obj, basicOnly):
         """Sliders present the following information (examples include
         'Pitch slider, 5.0, 56%'; 'Volume slider, 9.0, 100%'):
 
@@ -337,7 +342,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakMenuItem(self, obj, doubleClick):
+    def _speakMenuItem(self, obj, basicOnly):
         """Menu items present the following information (examples include
         'File menu, Open..., Control + O, item 2 of 20, O', 'File menu,
         Wizards Menu, item 4 of 20, W'):
@@ -408,7 +413,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakPageTab(self, obj, doubleClick):
+    def _speakPageTab(self, obj, basicOnly):
         """Tabs in a Tab List present the following information (an example
         is 'Tab list, braille page, item 2 of 5'):
 
@@ -438,7 +443,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakText(self, obj, doubleClick):
+    def _speakText(self, obj, basicOnly):
         """Text boxes present the following information (an example is
         'Source display: text, blank, Alt O'):
 
@@ -455,6 +460,12 @@ class WhereAmI:
         Gaim, gedit, OpenOffice Writer and Terminal
         """
 
+        cell = self._script.getAncestor(obj,
+                                        [pyatspi.ROLE_TABLE_CELL],
+                                        [pyatspi.ROLE_FRAME])
+        if cell:
+            return self._speakTableCell(cell, basicOnly)
+
         utterances = []
         text = self._getObjLabel(obj)
         utterances.append(text)
@@ -463,8 +474,8 @@ class WhereAmI:
         utterances.append(text)
 
         [textContents, startOffset, endOffset, selected] = \
-                       self._getTextContents(obj, doubleClick)
-        if doubleClick:
+                       self._getTextContents(obj, basicOnly)
+        if not basicOnly:
             # Speak character attributes.
             textContents = \
                 self._insertAttributes(obj, startOffset,
@@ -478,7 +489,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-        if doubleClick:
+        if not basicOnly:
             settings.verbalizePunctuationStyle = savedStyle
 
         utterances = []
@@ -498,71 +509,85 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakTableCell(self, obj, doubleClick):
+    def _speakTableCell(self, obj, basicOnly):
         """Tree Tables present the following information (an example is
         'Tree table, Mike Pedersen, item 8 of 10, tree level 2'):
 
-        1. label, if any
-        2. role
-        3. current row (regardless of speak cell/row setting)
-        4. relative position
-        5. if expandable/collapsible: expanded/collapsed
-        6. if applicable, the level
+        1. parent's role
+        2. column header of object
+        3. row header of object
+        4. object's role
+        5. object's contents, if there are multiple columns
+        6. relative position
+        7. current row (regardless of speak cell/row setting), if
+           performing a detailed whereAmI.
+        8. if expandable/collapsible: expanded/collapsed
+        9. if applicable, the level
 
         Nautilus and Gaim
         """
 
-        # Speak the first two items (and possibly the position)
         utterances = []
         if obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
             obj = obj.parent
         parent = obj.parent
 
-        text = self._getObjLabel(obj)
+        text = rolenames.getSpeechForRoleName(parent)
         utterances.append(text)
 
+        try:
+            table = parent.queryTable()
+        except:
+            table = None
+            nColumns = 0
+        else:
+            nColumns = table.nColumns
+            column = table.getColumnAtIndex(obj.getIndexInParent())
+            header = table.getColumnHeader(column)
+            if header:
+                text = self._getObjName(header)
+                utterances.append(text)
+
+            row = table.getRowAtIndex(obj.getIndexInParent())
+            header = table.getRowHeader(row)
+            if header:
+                text = self._getObjName(header)
+                utterances.append(text)
+
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
+
+        if nColumns:
+            text = self._getTableCell(obj)
+            utterances.append(text)
+
         debug.println(self._debugLevel, "first table cell utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
 
         utterances = []
-        if doubleClick:
-            table = parent.queryTable()
-            row = table.getRowAtIndex(
-              orca_state.locusOfFocus.getIndexInParent())
-            # Translators: this in reference to a row in a table.
+        if table:
+            # Translators: this is in references to a column in a
+            # table.
+            text = _("column %d of %d") % ((column + 1), table.nColumns)
+            utterances.append(text)
+
+            # Translators: this is in reference to a row in a table.
             #
-            text = _("row %d of %d") % ((row+1), table.nRows)
+            text = _("row %d of %d") % ((row + 1), table.nRows)
             utterances.append(text)
             speech.speakUtterances(utterances)
 
-        # Speak the current row
-        utterances = self._getTableRow(obj)
-        debug.println(self._debugLevel, "second table cell utterances=%s" % \
-                      utterances)
-        speech.speakUtterances(utterances)
-
-        # Speak the remaining items.
+            # Speak the current row if performing a "detailed" whereAmI.
+            #
+            if not basicOnly:
+                utterances = self._getTableRow(obj)
+                debug.println(self._debugLevel, \
+                              "second table cell utterances=%s" % \
+                              utterances)
+                speech.speakUtterances(utterances)
+        
         utterances = []
-
-        if not doubleClick:
-            try:
-                table = parent.queryTable()
-            except NotImplementedError:
-                debug.println(self._debugLevel, 
-                              "??? parent=%s" % parent.getRoleName())
-                return
-            else:
-                row = \
-                    table.getRowAtIndex(
-                       orca_state.locusOfFocus.getIndexInParent())
-                # Translators: this in reference to a row in a table.
-                #
-                text = _("row %d of %d") % ((row+1), table.nRows)
-                utterances.append(text)
-
         state = obj.getState()
         if state.contains(pyatspi.STATE_EXPANDABLE):
             if state.contains(pyatspi.STATE_EXPANDED):
@@ -590,7 +615,7 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakListItem(self, obj, doubleClick):
+    def _speakListItem(self, obj, basicOnly):
         """List items should be treated like tree cells:
 
         1. label, if any
@@ -651,12 +676,12 @@ class WhereAmI:
                       utterances)
         speech.speakUtterances(utterances)
 
-    def _speakParagraph(self, obj, doubleClick):
+    def _speakParagraph(self, obj, basicOnly):
         """Speak a paragraph object.
         """
-        self._speakText(obj, doubleClick)
+        self._speakText(obj, basicOnly)
 
-    def _speakIconPanel(self, obj, doubleClick):
+    def _speakIconPanel(self, obj, basicOnly):
         """Speak the contents of the pane containing this icon. The
         number of icons in the pane is spoken. Plus the total number of
         selected icons. Plus the name of each of the selected icons.
@@ -671,48 +696,17 @@ class WhereAmI:
 
         utterances.append(_("Icon panel"))
         utterances.append(self.getObjLabelAndName(obj))
-
-        selectedItems = []
-        totalSelectedItems = 0
-        currentItem = 0
-        for child in panel:
-            state = child.getState()
-            if state.contains(pyatspi.STATE_SELECTED):
-                totalSelectedItems += 1
-                selectedItems.append(child)
-            if state.contains(pyatspi.STATE_FOCUSED):
-                currentItem = child.getIndexInParent() + 1
-
-        # Translators: this is a count of the number of selected icons 
-        # and the count of the total number of icons within an icon panel. 
-        # An example of an icon panel is the Nautilus folder view.
-        #
-        itemString = ngettext("%d of %d item selected",
-                              "%d of %d items selected",
-                              childCount) % \
-                              (totalSelectedItems, childCount)
-        utterances.append(itemString)
-
-        # Translators: this is a indication of the focused icon and the
-        # count of the total number of icons within an icon panel. An
-        # example of an icon panel is the Nautilus folder view.
-        #
-        itemString = _("on item %d of %d") % (currentItem, childCount)
-        utterances.append(itemString)
-
-        if doubleClick:
-            for i in range(0, len(selectedItems)):
-                utterances.append(self.getObjLabelAndName(selectedItems[i]))
+        utterances.extend(self._getSelectedItemCount(panel, basicOnly))
 
         speech.speakUtterances(utterances)
         
-    def _speakLink(self, obj, doubleClick):
+    def _speakLink(self, obj, basicOnly):
         """Speaks information about a link including protocol, domain 
         comparisons and size of file if possible
         
         Arguments:
         - obj: the icon object that currently has focus.
-        - doubleClick: was it a doubleclick event?
+        - basicOnly: True if the user is performing a standard/basic whereAmI.
         """
         
         # get the URI for the link of interest and parse it.
@@ -772,7 +766,7 @@ class WhereAmI:
 
         speech.speakUtterances([linkoutput, domainoutput, sizeoutput])
 
-    def _speakToggleButton(self, obj, doubleClick):
+    def _speakToggleButton(self, obj, basicOnly):
         """Speak toggle button information:
            1. Name/Label
            2. Role
@@ -798,6 +792,133 @@ class WhereAmI:
 
         speech.speakUtterances(utterances)
       
+    def _speakSplitPane(self, obj, basicOnly):
+        """Speak split pane information:
+           1. Name/Label
+           2. Role
+           3. Value
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj)
+        utterances.append(text)
+
+        text = rolenames.getSpeechForRoleName(obj)
+        utterances.append(text)
+
+        valueString = self._script.getTextForValue(obj)
+        utterances.append(valueString)
+
+        speech.speakUtterances(utterances)
+
+    def _speakLabel(self, obj, basicOnly):
+        """Speak label information:
+           1. Name/Label
+           2. selected (if True).
+           3. Role
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj)
+        utterances.append(text)
+
+        utterances.extend(self._getSpeechForAllTextSelection(obj))
+
+        text = rolenames.getSpeechForRoleName(obj)
+        utterances.append(text)
+        speech.speakUtterances(utterances)
+
+    def _speakLayeredPane(self, obj, basicOnly):
+        """Speak layered pane information:
+           1. Name/Label
+           2. Role
+           3. Number of selected items and total number of items.
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj)
+        utterances.append(text)
+
+        text = rolenames.getSpeechForRoleName(obj)
+        utterances.append(text)
+        utterances.extend(self._getSelectedItemCount(obj, basicOnly))
+
+        speech.speakUtterances(utterances)
+
+    def _getSpeechForAllTextSelection(self, obj):
+        """Check if this object has text associated with it and it's
+        completely selected.
+
+        Arguments:
+        - obj: the object being presented
+        """
+
+        utterance = []
+        try:
+            textObj = obj.queryText()
+        except:
+            pass
+        else:
+            noOfSelections = textObj.getNSelections()
+            if noOfSelections == 1:
+                [string, startOffset, endOffset] = \
+                   textObj.getTextAtOffset(0, pyatspi.TEXT_BOUNDARY_LINE_START)
+                if startOffset == 0 and endOffset == len(string):
+                    # Translators: when the user selects (highlights) text in
+                    # a document, Orca lets them know this.
+                    #
+                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                    #
+                    utterance = [Q_("text|selected")]
+
+        return utterance
+
+    def _getSelectedItemCount(self, obj, basicOnly):
+        """Return an utterance indicating how many items are selected in this
+        object, the current item, and (if a detailed whereAmI), the names of
+        all the selected items. This object will be an icon panel or a 
+        layered pane.
+
+        Arguments:
+        - obj: the object being presented
+        """
+
+        childCount = obj.childCount
+        selectedItems = []
+        totalSelectedItems = 0
+        currentItem = 0
+        for child in obj:
+            state = child.getState()
+            if state.contains(pyatspi.STATE_SELECTED):
+                totalSelectedItems += 1
+                selectedItems.append(child)
+            if state.contains(pyatspi.STATE_FOCUSED):
+                currentItem = child.getIndexInParent() + 1
+
+        utterances = []
+        # Translators: this is a count of the number of selected icons
+        # and the count of the total number of icons within an icon panel.
+        # An example of an icon panel is the Nautilus folder view.
+        #
+        countString = ngettext("%d of %d item selected",
+                              "%d of %d items selected",
+                              childCount) % \
+                              (totalSelectedItems, childCount)
+        utterances.append(countString)
+
+        # Translators: this is a indication of the focused icon and the
+        # count of the total number of icons within an icon panel. An
+        # example of an icon panel is the Nautilus folder view.
+        #
+        itemString = _("on item %d of %d") % (currentItem, childCount)
+        utterances.append(itemString)
+
+        if not basicOnly:
+            for i in range(0, len(selectedItems)):
+                utterances.append(self.getObjLabelAndName(selectedItems[i]))
+
+        return utterances
+
     def __extractSize(self, uri):
         """Get the http header for a given uri and try to extract the size
         (Content-length).
@@ -832,7 +953,7 @@ class WhereAmI:
             #
             return _('%.2f megabytes') % (float(size) * .000001) 
 
-    def _speakGenericObject(self, obj, doubleClick):
+    def _speakGenericObject(self, obj, basicOnly):
         """Speak a generic object; one not specifically handled by
         other methods.
         """
@@ -1085,7 +1206,11 @@ class WhereAmI:
             pass
         else:
             for i in range(0, action.nActions):
-                if action.getName(i) == "toggle":
+                # Translators: this is the action name for
+                # the 'toggle' action. It must be the same
+                # string used in the *.po file for gail.
+                #
+                if action.getName(i) in ["toggle", _("toggle")]:
                     isToggle = True
                     break
 
@@ -1112,7 +1237,11 @@ class WhereAmI:
                 pass
             else:
                 for i in range(0, action.nActions):
-                    if action.getName(i) == "toggle":
+                    # Translators: this is the action name for
+                    # the 'toggle' action. It must be the same
+                    # string used in the *.po file for gail.
+                    #
+                    if action.getName(i) in ["toggle", _("toggle")]:
                         isToggle = True
                         break
         if not isToggle:
@@ -1164,15 +1293,15 @@ class WhereAmI:
 
         return [textContents, startOffset, endOffset]
 
-    def getTextSelections(self, obj, doubleClick):
+    def getTextSelections(self, obj, basicOnly):
         """Get all the text applicable text selections for the given object.
-        If the user doubleclicked, look to see if there are any previous 
-        or next text objects that also have selected text and add in their 
-        text contents.
+        If the user is doing a detailed whereAmI, look to see if there are
+        any previous or next text objects that also have selected text and
+        add in their text contents.
 
         Arguments:
         - obj: the text object to start extracting the selected text from.
-        - doubleClick: True if the user double-clicked the "where am I" key.
+        - basicOnly: True if the user is performing a standard/basic whereAmI.
 
         Returns: all the selected text contents plus the start and end
         offsets within the text for the given object.
@@ -1186,7 +1315,7 @@ class WhereAmI:
             [textContents, startOffset, endOffset] = \
                                             self._getTextSelection(obj)
 
-        if doubleClick:
+        if not basicOnly:
             current = obj
             morePossibleSelections = True
             while morePossibleSelections:
@@ -1308,7 +1437,7 @@ class WhereAmI:
 
         return [currentSelected, otherSelected]
 
-    def _getTextContents(self, obj, doubleClick):
+    def _getTextContents(self, obj, basicOnly):
         """Returns utterences for text.
 
         A. if no text on the current line is selected, the current line
@@ -1328,11 +1457,11 @@ class WhereAmI:
             (caretOffset, nSelections))
 
         [current, other] = self._hasTextSelections(obj)
-        if (doubleClick and (current or other)) or \
-           (not doubleClick and current):
+        if (not basicOnly and (current or other)) or \
+           (basicOnly and current):
             selected = True
             [textContents, startOffset, endOffset] = \
-                                  self.getTextSelections(obj, doubleClick)
+                                  self.getTextSelections(obj, basicOnly)
         else:
             # Get the line containing the caret
             #
@@ -1455,43 +1584,51 @@ class WhereAmI:
 
         return dictionary
 
-    def _speakTitleOrStatus(self, obj, doubleClick):
-        """When pressed a single time, Orca will speak the following
-        information:
+    def speakTitle(self, obj):
+        """Orca will speak the following information:
 
         1. The contents of the title bar of the application main window
         2. If in a dialog box within an application, the contents of the
         title bar of the dialog box.
         3. Orca will pause briefly between these two pieces of information
         so that the speech user can distinguish each.
-
-        If pressed twice, Orca will speak the status bar.
         """
 
         utterances = []
 
         results = self._getFrameAndDialog(obj)
 
-        if doubleClick:
-            if results[0]:
-                self._statusBar = None
-                self._getStatusBar(results[0])
-                if self._statusBar:
-                    self._speakStatusBar()
-            window = results[1] or results[0]
-            if window:
-                self._speakDefaultButton(window)
-        else:
-            if results[0]:
-                text = self.getObjLabelAndName(results[0])
-                utterances.append(text)
-            if results[1]:
-                text = self.getObjLabelAndName(results[1])
-                utterances.append(text)
+        if results[0]:
+            text = self.getObjLabelAndName(results[0])
+            utterances.append(text)
+        if results[1]:
+            text = self.getObjLabelAndName(results[1])
+            utterances.append(text)
 
-            debug.println(self._debugLevel, "titlebar utterances=%s" % \
-                          utterances)
-            speech.speakUtterances(utterances)
+        debug.println(self._debugLevel, "titlebar utterances=%s" % \
+                      utterances)
+        speech.speakUtterances(utterances)
+
+    def speakStatusBar(self, obj):
+        """Speak the contents of the status bar of the window with focus.
+        """
+ 
+        utterances = []
+ 
+        results = self._getFrameAndDialog(obj)
+
+        if results[0]:
+            self._statusBar = None
+            self._getStatusBar(results[0])
+            if self._statusBar:
+                self._speakStatusBar()
+        window = results[1] or results[0]
+        if window:
+            self._speakDefaultButton(window)
+
+        debug.println(self._debugLevel, "status bar utterances=%s" % \
+                      utterances)
+        speech.speakUtterances(utterances)
 
     def _getFrameAndDialog(self, obj):
         """Returns the frame and (possibly) the dialog containing
